package com.sun.java.util.jar.pack;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.io.InputStream;
import java.util.HashMap;
import java.io.IOException;
import java.util.Properties;
import java.util.Comparator;
import java.util.Set;
import java.io.PrintWriter;
import java.io.PrintStream;
import java.util.Collection;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.Map;
import java.util.List;
import java.util.TreeMap;
import java.util.SortedMap;

final class PropMap implements SortedMap<String, String>
{
    private final TreeMap<String, String> theMap;
    private final List<Object> listenerList;
    private static Map<String, String> defaultProps;
    
    void addListener(Object o) {
        assert !(!PropMap.Beans.isPropertyChangeListener(o));
        this.listenerList.add(o);
        return;
    }
    
    void removeListener(Object o) {
        assert !(!PropMap.Beans.isPropertyChangeListener(o));
        this.listenerList.remove(o);
        return;
    }
    
    @Override
    public String put(String key, String value) {
        String s = (String)this.theMap.put(key, value);
        if (value != s && !this.listenerList.isEmpty()) {
            assert !(!PropMap.Beans.isBeansPresent());
            Object propertyChangeEvent = PropMap.Beans.newPropertyChangeEvent(this, key, s, value);
            Iterator<Object> iterator = this.listenerList.iterator();
            while (iterator.hasNext()) {
                PropMap.Beans.invokePropertyChange(iterator.next(), propertyChangeEvent);
            }
        }
        return s;
    }
    
    PropMap() {
        this.theMap = new TreeMap();
        this.listenerList = new ArrayList(1);
        this.theMap.putAll(PropMap.defaultProps);
    }
    
    SortedMap<String, String> prefixMap(String s) {
        int length = s.length();
        if (length == 0) {
            return this;
        }
        return this.subMap(s, s.substring(0, length - 1) + (char)(s.charAt(length - 1) + 1));
    }
    
    String getProperty(String s) {
        return this.get(s);
    }
    
    String getProperty(String s, String s2) {
        String property = this.getProperty(s);
        if (property == null) {
            return s2;
        }
        return property;
    }
    
    String setProperty(String s, String s2) {
        return this.put(s, s2);
    }
    
    List<String> getProperties(String s) {
        Collection<String> values = this.prefixMap(s).values();
        ArrayList<Object> list = new ArrayList(values.size());
        list.addAll(values);
        while (list.remove(null)) {}
        return list;
    }
    
    private boolean toBoolean(String s) {
        return (boolean)Boolean.valueOf(s);
    }
    
    boolean getBoolean(String s) {
        return this.toBoolean(this.getProperty(s));
    }
    
    boolean setBoolean(String s, boolean b) {
        return this.toBoolean(this.setProperty(s, String.valueOf(b)));
    }
    
    int toInteger(String s) {
        return this.toInteger(s, 0);
    }
    
    int toInteger(String s, int n) {
        if (s == null) {
            return n;
        }
        if ("true".equals(s)) {
            return 1;
        }
        if ("false".equals(s)) {
            return 0;
        }
        return Integer.parseInt(s);
    }
    
    int getInteger(String s, int n) {
        return this.toInteger(this.getProperty(s), n);
    }
    
    int getInteger(String s) {
        return this.toInteger(this.getProperty(s));
    }
    
    int setInteger(String s, int i) {
        return this.toInteger(this.setProperty(s, String.valueOf(i)));
    }
    
    long toLong(String s) {
        try {
            return (s == null) ? 0L : Long.parseLong(s);
        }
        catch (NumberFormatException ex) {
            throw new IllegalArgumentException("Invalid value");
        }
    }
    
    long getLong(String s) {
        return this.toLong(this.getProperty(s));
    }
    
    long setLong(String s, long l) {
        return this.toLong(this.setProperty(s, String.valueOf(l)));
    }
    
    int getTime(String s) {
        String property = this.getProperty(s, "0");
        if ("now".equals(property)) {
            return (int)((System.currentTimeMillis() + 500L) / 1000L);
        }
        long long1 = this.toLong(property);
        if (long1 < 10000000000L && !"0".equals(property)) {
            Utils.log.warning("Supplied modtime appears to be seconds rather than milliseconds: " + property);
        }
        return (int)((long1 + 500L) / 1000L);
    }
    
    void list(PrintStream out) {
        PrintWriter printWriter = new PrintWriter(out);
        this.list(printWriter);
        printWriter.flush();
    }
    
    void list(PrintWriter printWriter) {
        printWriter.println("#PACK200[");
        Set<Map.Entry<String, String>> entrySet = PropMap.defaultProps.entrySet();
        for (Map.Entry entry : this.theMap.entrySet()) {
            if (entrySet.contains(entry)) {
                continue;
            }
            printWriter.println("  " + (String)entry.getKey() + " = " + (String)entry.getValue());
        }
        printWriter.println("#]");
    }
    
    @Override
    public int size() {
        return this.theMap.size();
    }
    
    @Override
    public boolean isEmpty() {
        return this.theMap.isEmpty();
    }
    
    @Override
    public boolean containsKey(Object key) {
        return this.theMap.containsKey(key);
    }
    
    @Override
    public boolean containsValue(Object value) {
        return this.theMap.containsValue(value);
    }
    
    @Override
    public String get(Object key) {
        return (String)this.theMap.get(key);
    }
    
    @Override
    public String remove(Object key) {
        return (String)this.theMap.remove(key);
    }
    
    @Override
    public void putAll(Map<? extends String, ? extends String> map) {
        this.theMap.putAll(map);
    }
    
    @Override
    public void clear() {
        this.theMap.clear();
    }
    
    @Override
    public Set<String> keySet() {
        return this.theMap.keySet();
    }
    
    @Override
    public Collection<String> values() {
        return this.theMap.values();
    }
    
    @Override
    public Set<Map.Entry<String, String>> entrySet() {
        return this.theMap.entrySet();
    }
    
    @Override
    public Comparator<? super String> comparator() {
        return this.theMap.comparator();
    }
    
    @Override
    public SortedMap<String, String> subMap(String fromKey, String toKey) {
        return this.theMap.subMap(fromKey, toKey);
    }
    
    @Override
    public SortedMap<String, String> headMap(String toKey) {
        return this.theMap.headMap(toKey);
    }
    
    @Override
    public SortedMap<String, String> tailMap(String fromKey) {
        return this.theMap.tailMap(fromKey);
    }
    
    @Override
    public String firstKey() {
        return (String)this.theMap.firstKey();
    }
    
    @Override
    public String lastKey() {
        return (String)this.theMap.lastKey();
    }
    
    static {
        Properties m = new Properties();
        m.put("com.sun.java.util.jar.pack.disable.native", String.valueOf(Boolean.getBoolean("com.sun.java.util.jar.pack.disable.native")));
        m.put("com.sun.java.util.jar.pack.verbose", String.valueOf(Integer.getInteger("com.sun.java.util.jar.pack.verbose", 0)));
        m.put("com.sun.java.util.jar.pack.default.timezone", String.valueOf(Boolean.getBoolean("com.sun.java.util.jar.pack.default.timezone")));
        m.put("pack.segment.limit", "-1");
        m.put("pack.keep.file.order", "true");
        m.put("pack.modification.time", "keep");
        m.put("pack.deflate.hint", "keep");
        m.put("pack.unknown.attribute", "pass");
        m.put("com.sun.java.util.jar.pack.class.format.error", System.getProperty("com.sun.java.util.jar.pack.class.format.error", "pass"));
        m.put("pack.effort", "5");
        String s = "intrinsic.properties";
        try (final InputStream resourceAsStream = PackerImpl.class.getResourceAsStream(s)) {
            if (resourceAsStream == null) {
                throw new RuntimeException(s + " cannot be loaded");
            }
            m.load(resourceAsStream);
        }
        catch (IOException cause) {
            throw new RuntimeException(cause);
        }
        for (Map.Entry entry : m.entrySet()) {
            String s2 = (String)entry.getKey();
            String s3 = (String)entry.getValue();
            if (s2.startsWith("attribute.")) {
                entry.setValue(Attribute.normalizeLayoutString(s3));
            }
            else {
                continue;
            }
        }
        PropMap.defaultProps = new HashMap(m);
    }
    
    private static class Beans
    {
        private static final Class<?> propertyChangeListenerClass;
        private static final Class<?> propertyChangeEventClass;
        private static final Method propertyChangeMethod;
        private static final Constructor<?> propertyEventCtor;
        
        private static Class<?> getClass(String name) {
            try {
                return Class.forName(name, true, PropMap.Beans.class.getClassLoader());
            }
            catch (ClassNotFoundException ex) {
                return null;
            }
        }
        
        private static Constructor<?> getConstructor(Class<?> clazz, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getDeclaredConstructor(parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        private static Method getMethod(Class<?> clazz, String name, Class<?>... parameterTypes) {
            try {
                return (clazz == null) ? null : clazz.getMethod(name, parameterTypes);
            }
            catch (NoSuchMethodException detailMessage) {
                throw new AssertionError(detailMessage);
            }
        }
        
        static boolean isBeansPresent() {
            return PropMap.Beans.propertyChangeListenerClass != null && PropMap.Beans.propertyChangeEventClass != null;
        }
        
        static boolean isPropertyChangeListener(Object o) {
            return PropMap.Beans.propertyChangeListenerClass != null && PropMap.Beans.propertyChangeListenerClass.isInstance(o);
        }
        
        static Object newPropertyChangeEvent(Object o, String s, Object o2, Object o3) {
            try {
                return PropMap.Beans.propertyEventCtor.newInstance(new Object[] { o, s, o2, o3 });
            }
            catch (InstantiationException | IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException detailMessage2) {
                Throwable cause = detailMessage2.getCause();
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                throw new AssertionError(detailMessage2);
            }
        }
        
        static void invokePropertyChange(Object obj, Object o) {
            try {
                PropMap.Beans.propertyChangeMethod.invoke(obj, new Object[] { o });
            }
            catch (IllegalAccessException detailMessage) {
                throw new AssertionError(detailMessage);
            }
            catch (InvocationTargetException detailMessage2) {
                Throwable cause = detailMessage2.getCause();
                if ((cause instanceof Error)) {
                    throw (Error)cause;
                }
                if ((cause instanceof RuntimeException)) {
                    throw (RuntimeException)cause;
                }
                throw new AssertionError(detailMessage2);
            }
        }
        
        static {
            propertyChangeListenerClass = getClass("java.beans.PropertyChangeListener");
            propertyChangeEventClass = getClass("java.beans.PropertyChangeEvent");
            propertyChangeMethod = getMethod(PropMap.Beans.propertyChangeListenerClass, "propertyChange", new Class[] { PropMap.Beans.propertyChangeEventClass });
            propertyEventCtor = getConstructor(PropMap.Beans.propertyChangeEventClass, new Class[] { Object.class, String.class, Object.class, Object.class });
        }
    }
}
